@@ -1,22 +1,22 @@
 BUILD_BIT='64bit'
 BUILD_CPU='unknown'
-BUILD_DATE='2018-07-17'
+BUILD_DATE='2018-10-02'
 BUILD_MACHINE='aarch64'
 BUILD_OS='Linux   '
-BUILD_TIME='14:33'
+BUILD_TIME='11:23'
 BUILT_BY='builduser'
 BUILT_ON='leming'
-COMPILER_VERSION='aarch64-unknown-linux-gnu-gcc (GCC) 8.1.1 20180531'
-CYTHON_VERSION='0.28.4'
-LINKER_VERSION='GNU ld (GNU Binutils) 2.30'
+COMPILER_VERSION='aarch64-unknown-linux-gnu-gcc (GCC) 8.2.0'
+CYTHON_VERSION='0.28.5'
+LINKER_VERSION='GNU ld (GNU Binutils) 2.31.1'
 PYTHON_VERSION='2.7.15'
 RELEASE_BUILD=True
 lib_pycairo='1.17.0'
 lib_pygobject='2.28.7'
 lib_pygtk='2.24.0'
 lib_vpx='1.7.0'
-lib_x11='1.6.5'
-lib_x264='0.152.2854M e9a5903'
+lib_x11='1.6.6'
+lib_x264='0.155.2917M 0a84d98'
 lib_x265='2.8'
 lib_xcomposite='0.4.4'
 lib_xdamage='1.1.4'
