@@ -201,7 +201,6 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
         self.show_pointer_overlay_timer = None
         self.moveresize_timer = None
         self.moveresize_event = None
-        self.window_offset = None   #actual vs reported coordinates
         #add platform hooks
         self.connect_after("realize", self.on_realize)
         self.connect('unrealize', self.on_unrealize)
@@ -1278,14 +1277,16 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
             return
         prev = b.pointer_overlay
         if pos is None:
+            if prev is None:
+                return
             value = None
         else:
+            if prev and prev[:2]==pos[:2]:
+                return
             #store both scaled and unscaled value:
             #(the opengl client uses the raw value)
             value = pos[:2]+self._client.sp(*pos[:2])+pos[2:]
         mouselog("show_pointer_overlay(%s) previous value=%s, new value=%s", pos, prev, value)
-        if prev==value:
-            return
         b.pointer_overlay = value
         if not self.show_pointer_overlay_timer:
             self.show_pointer_overlay_timer = self.timeout_add(10, self.do_show_pointer_overlay, prev)
@@ -1297,10 +1298,15 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
         b = self._backing
         if not b:
             return
+        def abs_coords(x, y, size):
+            if self.window_offset:
+                x += self.window_offset[0]
+                y += self.window_offset[1]
+            return x, y, size
         value = b.pointer_overlay
         if value:
             #repaint the scale value (in window coordinates):
-            x, y, size = value[2:5]
+            x, y, size = abs_coords(*value[2:5])
             self.queue_draw(x-size, y-size, size*2, size*2)
             #clear it shortly after:
             self.cancel_remove_pointer_overlay_timer()
@@ -1309,7 +1315,7 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
                 self.show_pointer_overlay(None)
             self.remove_pointer_overlay_timer = self.timeout_add(CURSOR_IDLE_TIMEOUT*1000, remove_pointer_overlay)
         if prev:
-            px, py, psize = prev[2:5]
+            px, py, psize = abs_coords(*prev[2:5])
             self.queue_draw(px-psize, py-psize, psize*2, psize*2)
 
 
@@ -1486,17 +1492,27 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
             context.restore()
 
 
-    def paint_spinner(self, context, area):
+    def paint_spinner(self, context, area=None):
         log("%s.paint_spinner(%s, %s)", self, context, area)
+        c = self._client
+        if not c:
+            return
+        ww, wh = self.get_size()
+        w = c.cx(ww)
+        h = c.cy(wh)
         #add grey semi-opaque layer on top:
         context.set_operator(cairo.OPERATOR_OVER)
         context.set_source_rgba(0.2, 0.2, 0.2, 0.4)
-        context.rectangle(area)
-        #w, h = self._size
-        #context.rectangle(gdk.Rectangle(0, 0, w, h))
+        #we can't use the area as rectangle with:
+        #context.rectangle(area)
+        #because those would be unscaled dimensions
+        #it's easier and safer to repaint the whole window:
+        if is_gtk3():
+            context.rectangle(0, 0, w, h)
+        else:
+            context.rectangle(gdk.Rectangle(0, 0, w, h))
         context.fill()
         #add spinner:
-        w, h = self.get_size()
         dim = min(w/3.0, h/3.0, 100.0)
         context.set_line_width(dim/10.0)
         context.set_line_cap(cairo.LINE_CAP_ROUND)
@@ -1511,7 +1527,8 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
             context.stroke()
 
     def spinner(self, ok):
-        if not self.can_have_spinner():
+        c = self._client
+        if not self.can_have_spinner() or not c:
             return
         #with normal windows, we just queue a draw request
         #and let the expose event paint the spinner
@@ -1569,7 +1586,8 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
         if self._backing is None:
             #we may have cleared the backing, so we must re-create one:
             self._set_backing_size(w, h)
-        self.idle_add(self._focus_change, "initial")
+        if not self._override_redirect:
+            self.idle_add(self._focus_change, "initial")
 
     def get_window_frame_size(self):
         frame = self._client.get_frame_extents(self)
@@ -1653,7 +1671,7 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
 
     def resize(self, w, h, resize_counter=0):
         ww, wh = self.get_size()
-        geomlog("resize(%s, %s, %s) current size=%s, fullscreen=%s", w, h, resize_counter, (ww, wh), self._fullscreen)
+        geomlog("resize(%s, %s, %s) current size=%s, fullscreen=%s, maximized=%s", w, h, resize_counter, (ww, wh), self._fullscreen, self._maximized)
         self._resize_counter = resize_counter
         if (w, h)==(ww, wh):
             self._backing.offsets = 0, 0, 0, 0
@@ -1661,6 +1679,7 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
             return
         if not self._fullscreen and not self._maximized:
             gtk.Window.resize(self, w, h)
+            ww, wh = w, h
             self._backing.offsets = 0, 0, 0, 0
         else:
             self.center_backing(w, h)
