@@ -170,8 +170,11 @@ def platform_release(release):
         SYSTEMVERSION_PLIST = "/System/Library/CoreServices/SystemVersion.plist"
         try:
             import plistlib
-            with open(SYSTEMVERSION_PLIST, "rb") as f:
-                pl = plistlib.load(f)           #@UndefinedVariable
+            if PYTHON2:
+                pl = plistlib.readPlist('/System/Library/CoreServices/SystemVersion.plist')
+            else:
+                with open(SYSTEMVERSION_PLIST, "rb") as f:
+                    pl = plistlib.load(f)           #@UndefinedVariable
             return pl['ProductUserVisibleVersion']
         except Exception as e:
             get_util_logger().debug("platform_release(%s)", release, exc_info=True)
@@ -432,11 +435,11 @@ def shellsub(s, subs={}):
 
 def osexpand(s, actual_username="", uid=0, gid=0, subs={}):
     def expanduser(s):
-        if len(actual_username)>0 and s.startswith("~/"):
+        if actual_username and s.startswith("~/"):
             #replace "~/" with "~$actual_username/"
             return os.path.expanduser("~%s/%s" % (actual_username, s[2:]))
         return os.path.expanduser(s)
-    if len(actual_username)>0:
+    if actual_username:
         HOME = os.path.expanduser("~%s" % actual_username)
     else:
         HOME = os.path.expanduser("~")
@@ -454,7 +457,7 @@ def osexpand(s, actual_username="", uid=0, gid=0, subs={}):
         if not OSX:
             from xpra.platform.xposix.paths import get_runtime_dir
             d["XDG_RUNTIME_DIR"] = os.environ.get("XDG_RUNTIME_DIR", get_runtime_dir())
-    if len(actual_username)>0:
+    if actual_username:
         d.update({
             "USERNAME"  : actual_username,
             "USER"      : actual_username,
