@@ -20,12 +20,12 @@ import traceback
 from xpra.platform.dotxpra import DotXpra
 from xpra.util import csv, envbool, envint, DEFAULT_PORT
 from xpra.exit_codes import EXIT_SSL_FAILURE, EXIT_SSH_FAILURE, EXIT_STR
-from xpra.os_util import get_util_logger, getuid, getgid, monotonic_time, setsid, bytestostr, WIN32, OSX, POSIX
+from xpra.os_util import get_util_logger, getuid, getgid, monotonic_time, setsid, bytestostr, WIN32, OSX, POSIX, PYTHON3
 from xpra.scripts.parsing import info, warn, error, \
     parse_vsock, parse_env, is_local, \
     fixup_defaults, validated_encodings, validate_encryption, do_parse_cmdline, show_sound_codec_help, \
     supports_shadow, supports_server, supports_proxy, supports_mdns
-from xpra.scripts.config import OPTION_TYPES, CLIENT_OPTIONS, NON_COMMAND_LINE_OPTIONS, CLIENT_ONLY_OPTIONS, START_COMMAND_OPTIONS, BIND_OPTIONS, PROXY_START_OVERRIDABLE_OPTIONS, OPTIONS_ADDED_SINCE_V1, \
+from xpra.scripts.config import OPTION_TYPES, CLIENT_OPTIONS, NON_COMMAND_LINE_OPTIONS, CLIENT_ONLY_OPTIONS, START_COMMAND_OPTIONS, BIND_OPTIONS, PROXY_START_OVERRIDABLE_OPTIONS, OPTIONS_ADDED_SINCE_V1, OPTIONS_COMPAT_NAMES, \
     InitException, InitInfo, InitExit, \
     fixup_options, dict_to_validated_config, \
     make_defaults_struct, parse_bool, has_sound_support, name_to_field
@@ -100,6 +100,14 @@ def main(script_file, cmdline):
             return 1
     finally:
         platform_clean()
+        try:
+            sys.stdout.close()
+        except:
+            pass
+        try:
+            sys.stderr.close()
+        except:
+            pass
 
 
 def configure_logging(options, mode):
@@ -607,6 +615,10 @@ def parse_display_name(error_cb, opts, display_name, session_name_lookup=False):
         ssh_cmd = ssh[0].lower()
         is_putty = ssh_cmd.endswith("plink") or ssh_cmd.endswith("plink.exe")
         desc["is_putty"] = is_putty
+        if is_putty:
+            #special env used by plink:
+            env = os.environ.copy()
+            env["PLINK_PROTOCOL"] = "ssh"
 
         username, password, host, ssh_port = parse_host_string(host, 22)
         if password and is_putty:
@@ -619,10 +631,6 @@ def parse_display_name(error_cb, opts, display_name, session_name_lookup=False):
             #grr why bother doing it different?
             desc["ssh-port"] = ssh_port
             if is_putty:
-                #special env used by plink:
-                env = os.environ.copy()
-                env["PLINK_PROTOCOL"] = "ssh"
-                desc["env"] = env
                 full_ssh += ["-P", str(ssh_port)]
             else:
                 full_ssh += ["-p", str(ssh_port)]
@@ -881,16 +889,9 @@ def connect_to(display_desc, opts=None, debug_cb=None, ssh_fail_cb=ssh_connect_f
             remote_cmd += "else echo \"no run-xpra command found\"; exit 1; fi"
             if INITENV_COMMAND:
                 remote_cmd = INITENV_COMMAND + ";" + remote_cmd
-            #putty gets confused if we wrap things in shell command:
-            if display_desc.get("is_putty", False):
-                cmd.append(remote_cmd)
-            else:
-                #openssh doesn't have this problem,
-                #and this gives us better compatibility with weird login shells
-                cmd.append("sh -c '%s'" % remote_cmd)
+            cmd.append("sh -c '%s'" % remote_cmd)
             if debug_cb:
                 debug_cb("starting %s tunnel" % str(cmd[0]))
-                #debug_cb("starting ssh: %s with kwargs=%s" % (str(cmd), kwargs))
             #non-string arguments can make Popen choke,
             #instead of lazily converting everything to a string, we validate the command:
             for x in cmd:
@@ -1193,7 +1194,8 @@ def ssl_wrap_socket_fn(opts, server_side=True):
             import binascii
             cadata = binascii.unhexlify(cadata)
         except:
-            pass
+            import base64
+            cadata = base64.b64decode(cadata)
 
     kwargs = {
               "server_side"             : server_side,
@@ -1407,6 +1409,8 @@ def get_client_app(error_cb, opts, extra_args, mode):
         app.hello_extra = {"connect" : False}
         app.start_new_session = sns
     else:
+        if PYTHON3 and POSIX and os.environ.get("GDK_BACKEND") is None:
+            os.environ["GDK_BACKEND"] = "x11"
         try:
             from xpra.platform.gui import init as gui_init
             gui_init()
@@ -1530,7 +1534,7 @@ def run_remote_server(error_cb, opts, args, mode, defaults):
         proxy_args = []
         if params.get("display") is not None:
             proxy_args.append(params["display"])
-        for x in get_start_server_args(opts, True):
+        for x in get_start_server_args(opts, compat=True):
             proxy_args.append(shellquote(x))
         params["display_as_args"] = proxy_args
         #and use a proxy subcommand to start the server:
@@ -1621,8 +1625,10 @@ def find_X11_displays(max_display_no=None, match_uid=None, match_gid=None):
                 warn("failure on %s: %s" % (socket_path, e))
     return displays
 
-def guess_X11_display(dotxpra, uid=getuid(), gid=getgid()):
+def guess_X11_display(dotxpra, current_display, uid=getuid(), gid=getgid()):
     displays = [":%s" % x for x in find_X11_displays(max_display_no=10, match_uid=uid, match_gid=gid)]
+    if current_display and current_display not in displays:
+        displays.append(current_display)
     if len(displays)!=1:
         #try without uid match:
         displays = [":%s" % x for x in find_X11_displays(max_display_no=10, match_gid=gid)]
@@ -1640,6 +1646,8 @@ def guess_X11_display(dotxpra, uid=getuid(), gid=getgid()):
         displays = list(set(displays)-set(xpra_displays))
         if len(displays)==0:
             raise InitExit(1, "could not detect any live plain X11 displays, only multiple xpra displays: %s" % csv(xpra_displays))
+    if current_display:
+        return current_display
     if len(displays)!=1:
         raise InitExit(1, "too many live X11 displays to choose from: %s" % csv(displays))
     return displays[0]
@@ -1701,7 +1709,7 @@ def start_server_subprocess(script_file, args, mode, opts, username="", uid=getu
                 #display_name was provided:
                 display_name = args[0]
             else:
-                display_name = guess_X11_display(dotxpra, uid, gid)
+                display_name = guess_X11_display(dotxpra, None, uid, gid)
             #we now know the display name, so add it:
             args = [display_name]
         opts.exit_with_client = True
@@ -1810,6 +1818,9 @@ def get_start_server_args(opts, uid=getuid(), gid=getgid(), compat=False):
         dv = getattr(defaults, fn)
         if ov==dv:
             continue    #same as the default
+        argname = "--%s=" % x
+        if compat:
+            argname = OPTIONS_COMPAT_NAMES.get(argname, argname)
         #lists are special cased depending on how OptionParse will be parsing them:
         if ftype==list:
             #warn("%s: %s vs %s\n" % (x, ov, dv))
@@ -1821,10 +1832,10 @@ def get_start_server_args(opts, uid=getuid(), gid=getgid(), compat=False):
                      ]:
                 #individual arguments (ie: "--start=xterm" "--start=gedit" ..)
                 for e in ov:
-                    args.append("--%s=%s" % (x, e))
+                    args.append("%s%s" % (argname, e))
             else:
                 #those can be specified as CSV: (ie: "--encodings=png,jpeg,rgb")
-                args.append("--%s=%s" % (x, ",".join(str(x) for x in ov)))
+                args.append("%s%s" % (argname, ",".join(str(v) for v in ov)))
         elif ftype==bool:
             if compat and x in ("exit-with-children", "use-display", "mmap-group"):
                 #older servers don't take a bool value for those options,
@@ -1832,9 +1843,9 @@ def get_start_server_args(opts, uid=getuid(), gid=getgid(), compat=False):
                 if ov:
                     args.append("--%s" % x)
             else:
-                args.append("--%s=%s" % (x, ["no", "yes"][int(ov)]))
+                args.append("%s%s" % (argname, ["no", "yes"][int(ov)]))
         elif ftype in (int, float, str):
-            args.append("--%s=%s" % (x, ov))
+            args.append("%s%s" % (argname, ov))
         else:
             raise InitException("unknown option type '%s' for '%s'" % (ftype, x))
     return args
@@ -2095,7 +2106,7 @@ def run_list_mdns(error_cb, extra_args):
                         print(" uuid=%s" % uuid)
                 print(" + %s endpoint on host %s, port %i, interface %s" % (mode, address, port, iface))
                 dstr = ""
-                if display.startswith(":"):
+                if display and display.startswith(":"):
                     dstr = display[1:]
                 uri = "%s/%s@%s:%s/%s" % (mode, username, address, port, dstr)
                 print("   \"%s\"" % uri)
