@@ -43,6 +43,9 @@ def add_when_ready(f):
 def add_cleanup(f):
     _cleanups.append(f)
 
+def insert_cleanup(f):
+    _cleanups.insert(0, f)
+
 
 def deadly_signal(signum, _frame):
     info("got deadly signal %s, exiting\n" % SIGNAMES.get(signum, signum))
@@ -168,8 +171,8 @@ def display_name_check(display_name):
 def close_gtk_display():
     # Close our display(s) first, so the server dying won't kill us.
     # (if gtk has been loaded)
-    gdk_mod = sys.modules.get("gdk")
-    if gdk_mod:
+    gdk_mod = sys.modules.get("gtk.gdk") or sys.modules.get("gi.repository.Gdk")
+    if gdk_mod and envbool("XPRA_CLOSE_GTK_DISPLAY", False):
         for d in gdk_mod.display_manager_get().list_displays():
             d.close()
 
@@ -401,7 +404,7 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
         else:
             from xpra.scripts.main import guess_X11_display
             dotxpra = DotXpra(opts.socket_dir, opts.socket_dirs)
-            display_name = guess_X11_display(dotxpra)
+            display_name = guess_X11_display(dotxpra, desktop_display)
     elif upgrading and len(extra_args)==0:
         display_name = guess_xpra_display(opts.socket_dir, opts.socket_dirs)
     else:
@@ -772,14 +775,14 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
         except:
             pass
 
+    kill_display = None
     if not proxying:
-        def close_display():
-            close_gtk_display()
+        add_cleanup(close_gtk_display)
+    if not proxying and not shadowing:
+        def kill_display():
             if xvfb_pid:
                 kill_xvfb(xvfb_pid)
-        add_cleanup(close_display)
-    else:
-        close_display = None
+        add_cleanup(kill_display)
 
     if opts.daemon:
         def noerr(fn, *args):
@@ -901,8 +904,9 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
         if not check_xvfb():
             return  1
         assert starting or starting_desktop or upgrading
-        from xpra.x11.gtk2.gdk_display_source import init_gdk_display_source
+        from xpra.x11.gtk2.gdk_display_source import init_gdk_display_source, close_gdk_display_source
         init_gdk_display_source()
+        insert_cleanup(close_gdk_display_source)
         #(now we can access the X11 server)
 
         #make sure the pid we save is the real one:
@@ -1060,8 +1064,8 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
         r = -128
     if r>0:
         # Upgrading/exiting, so leave X and dbus servers running
-        if close_display:
-            _cleanups.remove(close_display)
+        if kill_display:
+            _cleanups.remove(kill_display)
         if kill_dbus:
             _cleanups.remove(kill_dbus)
         from xpra.server import EXITING_CODE
@@ -1071,4 +1075,9 @@ def run_server(error_cb, opts, mode, xpra_file, extra_args, desktop_display=None
             log.info("upgrading: not cleaning up Xvfb")
         log("cleanups=%s", _cleanups)
         r = 0
-    return r
+    try:
+        return r
+    finally:
+        run_cleanups()
+        import gc
+        gc.collect()
